/* Code generated by cbindgen. DO NOT EDIT. */

#ifndef WARDSTONE_H_
#define WARDSTONE_H_

#include <stdint.h>

/**
 * The level of security of a symmetric cryptosystem which is a
 * standard measure used to assess the security of all other
 * cryptographic primitives.
 */
typedef uint16_t ws_security;

/**
 * Represents the context in which a cryptographic primitive will be
 * assessed against such as the year and minimum security required by
 * the user.
 */
typedef struct ws_context {
  ws_security security;
  uint16_t year;
} ws_context;

/**
 * Represents an elliptic curve cryptography primitive used for digital
 * signatures and key establishment where f is the key size (the size
 * of n, where n is the order of the base point G).
 */
typedef struct ws_ecc {
  uint16_t id;
  uint16_t f;
} ws_ecc;

/**
 * Represents a finite field cryptography primitive used to implement
 * discrete logarithm cryptography.
 *
 * The choices l and n represents the bit lengths of the prime modulus
 * p and the prime divisor q.
 *
 * Some of the primitives that fall under this category include
 * signature algorithms such as DSA and key establishment algorithms
 * such as Diffie-Hellman and MQV.
 */
typedef struct ws_ffc {
  uint16_t id;
  uint16_t l;
  uint16_t n;
} ws_ffc;

/**
 * Represents a hash or hash-based function cryptographic primitive
 * where `id` is a unique identifier and `n` the digest length.
 */
typedef struct ws_hash {
  uint16_t id;
  uint16_t n;
} ws_hash;

/**
 * Represents an integer factorisation cryptography primitive the most
 * common of which is the RSA signature algorithm where k indicates the
 * key size.
 */
typedef struct ws_ifc {
  uint16_t id;
  uint16_t k;
} ws_ifc;

/**
 * Represents a symmetric key cryptography primitive.
 */
typedef struct ws_symmetric {
  uint16_t id;
  uint16_t security;
} ws_symmetric;

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

extern const struct ws_ecc WS_B163;

extern const struct ws_ecc WS_B233;

extern const struct ws_ecc WS_B283;

extern const struct ws_ecc WS_B409;

extern const struct ws_ecc WS_B571;

extern const struct ws_ecc WS_BRAINPOOLP160R1;

extern const struct ws_ecc WS_BRAINPOOLP160T1;

extern const struct ws_ecc WS_BRAINPOOLP192R1;

extern const struct ws_ecc WS_BRAINPOOLP192T1;

extern const struct ws_ecc WS_BRAINPOOLP224R1;

extern const struct ws_ecc WS_BRAINPOOLP224T1;

extern const struct ws_ecc WS_BRAINPOOLP256R1;

extern const struct ws_ecc WS_BRAINPOOLP256T1;

extern const struct ws_ecc WS_BRAINPOOLP320R1;

extern const struct ws_ecc WS_BRAINPOOLP320T1;

extern const struct ws_ecc WS_BRAINPOOLP384R1;

extern const struct ws_ecc WS_BRAINPOOLP384T1;

extern const struct ws_ecc WS_BRAINPOOLP512R1;

extern const struct ws_ecc WS_BRAINPOOLP512T1;

extern const struct ws_ecc WS_C2PNB163V1;

extern const struct ws_ecc WS_C2PNB163V2;

extern const struct ws_ecc WS_C2PNB163V3;

extern const struct ws_ecc WS_C2PNB176V1;

extern const struct ws_ecc WS_C2PNB208W1;

extern const struct ws_ecc WS_C2PNB272W1;

extern const struct ws_ecc WS_C2PNB304W1;

extern const struct ws_ecc WS_C2PNB368W1;

extern const struct ws_ecc WS_C2TNB191V1;

extern const struct ws_ecc WS_C2TNB191V2;

extern const struct ws_ecc WS_C2TNB191V3;

extern const struct ws_ecc WS_C2TNB239V1;

extern const struct ws_ecc WS_C2TNB239V2;

extern const struct ws_ecc WS_C2TNB239V3;

extern const struct ws_ecc WS_C2TNB359V1;

extern const struct ws_ecc WS_C2TNB431R1;

extern const struct ws_ecc WS_ED25519;

extern const struct ws_ecc WS_ED448;

extern const struct ws_ecc WS_K163;

extern const struct ws_ecc WS_K233;

extern const struct ws_ecc WS_K409;

extern const struct ws_ecc WS_K571;

extern const struct ws_ecc WS_P192;

extern const struct ws_ecc WS_P224;

extern const struct ws_ecc WS_P256;

extern const struct ws_ecc WS_P384;

extern const struct ws_ecc WS_P521;

extern const struct ws_ecc WS_PRIME192V1;

extern const struct ws_ecc WS_PRIME192V2;

extern const struct ws_ecc WS_PRIME192V3;

extern const struct ws_ecc WS_PRIME239V1;

extern const struct ws_ecc WS_PRIME239V2;

extern const struct ws_ecc WS_PRIME239V3;

extern const struct ws_ecc WS_PRIME256V1;

extern const struct ws_ecc WS_SECP112R1;

extern const struct ws_ecc WS_SECP112R2;

extern const struct ws_ecc WS_SECP128R1;

extern const struct ws_ecc WS_SECP128R2;

extern const struct ws_ecc WS_SECP160R1;

extern const struct ws_ecc WS_SECP160R2;

extern const struct ws_ecc WS_SECP192R1;

extern const struct ws_ecc WS_SECP192K1;

extern const struct ws_ecc WS_SECP224R1;

extern const struct ws_ecc WS_SECP224K1;

extern const struct ws_ecc WS_SECP256K1;

extern const struct ws_ecc WS_SECP256R1;

extern const struct ws_ecc WS_SECP384R1;

extern const struct ws_ecc WS_SECP521R1;

extern const struct ws_ecc WS_SECT113R1;

extern const struct ws_ecc WS_SECT113R2;

extern const struct ws_ecc WS_SECT131R1;

extern const struct ws_ecc WS_SECT131R2;

extern const struct ws_ecc WS_SECT163K1;

extern const struct ws_ecc WS_SECT163R1;

extern const struct ws_ecc WS_SECT163R2;

extern const struct ws_ecc WS_SECT193R1;

extern const struct ws_ecc WS_SECT193R2;

extern const struct ws_ecc WS_SECT233K1;

extern const struct ws_ecc WS_SECT233R1;

extern const struct ws_ecc WS_SECT239K1;

extern const struct ws_ecc WS_SECT283R1;

extern const struct ws_ecc WS_SECT409K1;

extern const struct ws_ecc WS_SECT409R1;

extern const struct ws_ecc WS_SECT571K1;

extern const struct ws_ecc WS_SECT571R1;

extern const struct ws_ecc WS_SM2;

extern const struct ws_ecc WS_WAP_WSG_IDM_ECID_WTLS1;

extern const struct ws_ecc WS_WAP_WSG_IDM_ECID_WTLS3;

extern const struct ws_ecc WS_WAP_WSG_IDM_ECID_WTLS4;

extern const struct ws_ecc WS_WAP_WSG_IDM_ECID_WTLS5;

extern const struct ws_ecc WS_WAP_WSG_IDM_ECID_WTLS6;

extern const struct ws_ecc WS_WAP_WSG_IDM_ECID_WTLS7;

extern const struct ws_ecc WS_WAP_WSG_IDM_ECID_WTLS8;

extern const struct ws_ecc WS_WAP_WSG_IDM_ECID_WTLS9;

extern const struct ws_ecc WS_WAP_WSG_IDM_ECID_WTLS10;

extern const struct ws_ecc WS_WAP_WSG_IDM_ECID_WTLS11;

extern const struct ws_ecc WS_WAP_WSG_IDM_ECID_WTLS12;

extern const struct ws_ecc WS_X25519;

extern const struct ws_ecc WS_X448;

extern const struct ws_ecc WS_ECC_224;

extern const struct ws_ecc WS_ECC_256;

extern const struct ws_ecc WS_ECC_384;

extern const struct ws_ecc WS_ECC_512;

extern const struct ws_ecc WS_ECC_NOT_ALLOWED;

extern const struct ws_ffc WS_DSA_1024_160;

extern const struct ws_ffc WS_DSA_2048_224;

extern const struct ws_ffc WS_DSA_2048_256;

extern const struct ws_ffc WS_DSA_3072_256;

extern const struct ws_ffc WS_DSA_7680_384;

extern const struct ws_ffc WS_DSA_15360_512;

extern const struct ws_ffc WS_FFC_NOT_SUPPORTED;

extern const struct ws_hash WS_BLAKE_224;

extern const struct ws_hash WS_BLAKE_256;

extern const struct ws_hash WS_BLAKE_384;

extern const struct ws_hash WS_BLAKE_512;

extern const struct ws_hash WS_BLAKE2B_256;

extern const struct ws_hash WS_BLAKE2B_384;

extern const struct ws_hash WS_BLAKE2B_512;

extern const struct ws_hash WS_BLAKE2S_256;

extern const struct ws_hash WS_BLAKE3;

extern const struct ws_hash WS_MD4;

extern const struct ws_hash WS_MD5;

extern const struct ws_hash WS_RIPEMD160;

extern const struct ws_hash WS_SHA1;

extern const struct ws_hash WS_SHA224;

extern const struct ws_hash WS_SHA256;

extern const struct ws_hash WS_SHA384;

extern const struct ws_hash WS_SHA3_224;

extern const struct ws_hash WS_SHA3_256;

extern const struct ws_hash WS_SHA3_384;

extern const struct ws_hash WS_SHA3_512;

extern const struct ws_hash WS_SHA512;

extern const struct ws_hash WS_SHA512_224;

extern const struct ws_hash WS_SHA512_256;

extern const struct ws_hash WS_SHAKE128;

extern const struct ws_hash WS_SHAKE256;

extern const struct ws_hash WS_WHIRLPOOL;

extern const uint16_t WS_ID_RSA_PKCS1;

extern const uint16_t WS_ID_RSA_PSS;

extern const struct ws_ifc WS_RSA_PKCS1_1024;

extern const struct ws_ifc WS_RSA_PKCS1_1536;

extern const struct ws_ifc WS_RSA_PKCS1_2048;

extern const struct ws_ifc WS_RSA_PKCS1_3072;

extern const struct ws_ifc WS_RSA_PKCS1_4096;

extern const struct ws_ifc WS_RSA_PKCS1_7680;

extern const struct ws_ifc WS_RSA_PKCS1_8192;

extern const struct ws_ifc WS_RSA_PKCS1_15360;

extern const struct ws_ifc WS_RSA_PSS_1024;

extern const struct ws_ifc WS_RSA_PSS_1280;

extern const struct ws_ifc WS_RSA_PSS_1536;

extern const struct ws_ifc WS_RSA_PSS_2048;

extern const struct ws_ifc WS_RSA_PSS_3072;

extern const struct ws_ifc WS_RSA_PSS_4096;

extern const struct ws_ifc WS_RSA_PSS_7680;

extern const struct ws_ifc WS_RSA_PSS_8192;

extern const struct ws_ifc WS_RSA_PSS_15360;

extern const struct ws_ifc WS_IFC_NOT_ALLOWED;

extern const struct ws_symmetric WS_AES128;

extern const struct ws_symmetric WS_AES192;

extern const struct ws_symmetric WS_AES256;

extern const struct ws_symmetric WS_CAMELLIA128;

extern const struct ws_symmetric WS_CAMELLIA192;

extern const struct ws_symmetric WS_CAMELLIA256;

extern const struct ws_symmetric WS_DES;

extern const struct ws_symmetric WS_DESX;

extern const struct ws_symmetric WS_IDEA;

extern const struct ws_symmetric WS_SERPENT128;

extern const struct ws_symmetric WS_SERPENT192;

extern const struct ws_symmetric WS_SERPENT256;

extern const struct ws_symmetric WS_TDEA2;

extern const struct ws_symmetric WS_TDEA3;

/**
 * Creates a context which will default to the year 2023 and will use
 * the minimum security defined by the standard.
 */
struct ws_context ws_context_default(void);

/**
 * Validate an elliptic curve cryptography primitive used for digital
 * signatures and key establishment where f is the key size.
 *
 * If the key is not compliant then `ws_ecc*` will contain the
 * recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `ws_ecc*` will also hold the recommended primitive with the
 * desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * **Note:** While the guide allows for elliptic curve system
 * parameters "that are provided by a trustworthy authority"
 * (see p. 73), this function conservatively deems any curve that is
 * not explicitly stated as non-compliant. This means only the
 * Brainpool curves are considered compliant.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_bsi_validate_ecc(struct ws_context ctx, struct ws_ecc key, struct ws_ecc *alternative);

/**
 * Validates a finite field cryptography primitive.
 *
 * Examples include the DSA and key establishment algorithms such as
 * Diffie-Hellman.
 *
 * If the key is not compliant then `struct ws_ffc*` will point to the
 * recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `struct ws_ffc` will also point to the recommended primitive
 * with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_bsi_validate_ffc(struct ws_context ctx, struct ws_ffc key, struct ws_ffc *alternative);

/**
 * Validates a hash function according to page 41 of the guide. The
 * reference is made with regards to applications that require
 * collision resistance such as digital signatures.
 *
 * For applications that primarily require pre-image resistance such as
 * message authentication codes (MACs), key derivation functions
 * (KDFs), and random bit generation use `ws_bsi_validate_hash_based`.
 *
 * If the hash function is not compliant then
 * `struct ws_hash* alternative` will point to the recommended
 * primitive that one should use instead.
 *
 * If the hash function is compliant but the context specifies a higher
 * security level, `struct ws_hash*` will also point to the recommended
 * primitive with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * **Caution:** Unlike the NIST standard, the guide does not make a
 * distinction between security requirements based on usage. For
 * example, collision resistance generally requires twice more the
 * security that one would want if they only cared about pre-image
 * resistance. As a result, this module does not have a corresponding
 * `validate_hash_based` function and the recommendation returned may
 * be overly conservative.
 *
 * **Note:** An alternative might be suggested for a compliant hash
 * function with a similar security level in which a switch to the
 * recommended primitive would likely be unwarranted. For example, when
 * evaluating compliance for the `SHA3-256`, a recommendation to use
 * `SHA256` will be made but switching to this as a result is likely
 * unnecessary.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_bsi_validate_hash(struct ws_context ctx, struct ws_hash hash, struct ws_hash *alternative);

/**
 * Validates a hash function. The reference is made with regards to
 * applications that primarily require pre-image resistance such as
 * message authentication codes (MACs), key derivation functions
 * (KDFs), and random bit generation.
 *
 * For applications that require collision resistance such digital
 * signatures use `ws_bsi_validate_hash`.
 *
 * If the hash function is not compliant then
 * `struct ws_hash* alternative` will point to the recommended
 * primitive that one should use instead.
 *
 * If the hash function is compliant but the context specifies a higher
 * security level, `struct ws_hash*` will also point to the recommended
 * primitive with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * **Note:** For an HMAC the minimum security required is ≥ 128 (see
 * p. 45) but the minimum digest length for a hash function that can be
 * used with this primitive is 256 (see p. 41). This means any
 * recommendation from this function will be likely too conservative.
 *
 * An alternative might also be suggested for a compliant hash
 * functions with a similar security level in which a switch to the
 * recommended primitive would likely be unwarranted. For example, when
 * evaluating compliance for the `SHA3-256`, a recommendation to use
 * `SHA256` will be made but switching to this as a result is likely
 * unnecessary.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_bsi_validate_hash_based(struct ws_context ctx,
                               struct ws_hash hash,
                               struct ws_hash *alternative);

/**
 * Validates  an integer factorisation cryptography primitive the most
 * common of which is the RSA signature algorithm.
 *
 * If the key is not compliant then `ws_ifc*` will point to the
 * recommended key size that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `ws_ifc*` will also point to the recommended key size with
 * the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 * **Note:** Unlike other functions in this module, this will return a
 * generic structure that specifies minimum private and public key
 * sizes.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_bsi_validate_ifc(struct ws_context ctx, struct ws_ifc key, struct ws_ifc *alternative);

/**
 * Validates a symmetric key primitive according to pages 24 of the
 * guide.
 *
 * If the key is not compliant then `struct ws_symmetric* alternative`
 * will point to the recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `struct ws_symmetric*` will also point to the recommended
 * primitive with the desired security level.
 *
 * The function returns `1` if the key is compliant, `0` if it is not,
 * and `-1` if an error occurs as a result of a missing or invalid
 * argument.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_bsi_validate_symmetric(struct ws_context ctx,
                              struct ws_symmetric key,
                              struct ws_symmetric *alternative);

/**
 * Validate an elliptic curve cryptography primitive used for digital
 * signatures and key establishment.
 *
 * If the key is not compliant then `ws_ecc*` will contain the
 * recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `ws_ecc*` will also hold the recommended primitive with the
 * desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_cnsa_validate_ecc(struct ws_context ctx, struct ws_ecc key, struct ws_ecc *alternative);

/**
 * Validates a finite field cryptography primitive function.
 *
 * Examples include the DSA and key establishment algorithms such as
 * Diffie-Hellman and MQV which can also be implemented as such.
 *
 * This primitive is not supported by either version of the CNSA
 * guidance.
 *
 * If the key is not compliant then `struct ws_ffc*` will point to the
 * recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `struct ws_ffc` will also point to the recommended primitive
 * with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_cnsa_validate_ffc(struct ws_context ctx, struct ws_ffc key, struct ws_ffc *alternative);

/**
 * Validates a hash function.
 *
 * Unlike other functions in this module, there is no distinction in
 * security based on the application. As such this module does not have
 * a corresponding `validate_hash_based` function. All hash function
 * and hash based application are assessed by this single function.
 *
 * If the hash function is not compliant then
 * `struct ws_hash* alternative` will point to the recommended
 * primitive that one should use instead.
 *
 * If the hash function is compliant but the context specifies a higher
 * security level, `struct ws_hash*` will also point to the recommended
 * primitive with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_cnsa_validate_hash(struct ws_context ctx, struct ws_hash hash, struct ws_hash *alternative);

/**
 * Validates  an integer factorisation cryptography primitive the most
 * common of which is the RSA signature algorithm.
 *
 * If the key is not compliant then `ws_ifc*` will point to the
 * recommended key size that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `ws_ifc*` will also point to the recommended key size with
 * the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 * **Note:** Unlike other functions in this module, this will return a
 * generic structure that specifies minimum private and public key
 * sizes.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_cnsa_validate_ifc(struct ws_context ctx, struct ws_ifc key, struct ws_ifc *alternative);

/**
 * Validates a symmetric key primitive.
 *
 * If the key is not compliant then `struct ws_symmetric* alternative`
 * will point to the recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `struct ws_symmetric*` will also point to the recommended
 * primitive with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_cnsa_validate_symmetric(struct ws_context ctx,
                               struct ws_symmetric key,
                               struct ws_symmetric *alternative);

/**
 * Validate an elliptic curve cryptography primitive used for digital
 * signatures and key establishment where f is the key size according
 * to page 47 of the report.
 *
 * If the key is not compliant then `ws_ecc*` will contain the
 * recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `ws_ecc*` will also hold the recommended primitive with the
 * desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * **Note:** This will return a generic structure that specifies key
 * sizes.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_ecrypt_validate_ecc(struct ws_context ctx, struct ws_ecc key, struct ws_ecc *alternative);

/**
 * Validates a finite field cryptography primitive according to page 47
 * of the report.
 *
 * Examples include the DSA and key establishment algorithms such as
 * Diffie-Hellman.
 *
 * If the key is not compliant then `struct ws_ffc*` will point to the
 * recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `struct ws_ffc` will also point to the recommended primitive
 * with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * **Note:** The choice of security specified in the `Context` is
 * restricted to the values 160, 224, 256, 384, and 512.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_ecrypt_validate_ffc(struct ws_context ctx, struct ws_ffc key, struct ws_ffc *alternative);

/**
 * Validates a hash function according to pages 40-43 of the report.
 *
 * Unlike other functions in this module, there is no distinction in
 * security based on the application. As such this module does not have
 * a corresponding `validate_hash_based` function. All hash function
 * and hash based application are assessed by this single function.
 *
 * If the hash function is not compliant then
 * `struct ws_hash* alternative` will point to the recommended
 * primitive that one should use instead.
 *
 * If the hash function is compliant but the context specifies a higher
 * security level, `struct ws_hash*` will also point to the recommended
 * primitive with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_ecrypt_validate_hash(struct ws_context ctx,
                            struct ws_hash hash,
                            struct ws_hash *alternative);

/**
 * Validates  an integer factorisation cryptography primitive the most
 * common of which is the RSA signature algorithm according to pages
 * 47-48.
 *
 * If the key is not compliant then `ws_ifc*` will point to the
 * recommended key size that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `ws_ifc*` will also point to the recommended key size with
 * the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 * **Note:** Unlike other functions in this module, this will return a
 * generic structure that specifies minimum private and public key
 * sizes.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_ecrypt_validate_ifc(struct ws_context ctx, struct ws_ifc key, struct ws_ifc *alternative);

/**
 * Validates a symmetric key primitive according to pages 37 to 40 of
 * the report.
 *
 * If the key is not compliant then `struct ws_symmetric* alternative`
 * will point to the recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `struct ws_symmetric*` will also point to the recommended
 * primitive with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_ecrypt_validate_symmetric(struct ws_context ctx,
                                 struct ws_symmetric key,
                                 struct ws_symmetric *alternative);

/**
 * Validate an elliptic curve cryptography primitive used for digital
 * signatures and key establishment where f is the key size.
 *
 * If the key is not compliant then `ws_ecc*` will contain the
 * recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `ws_ecc*` will also hold the recommended primitive with the
 * desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_lenstra_validate_ecc(struct ws_context ctx, struct ws_ecc key, struct ws_ecc *alternative);

/**
 * Validates a finite field cryptography primitive function examples
 * which include DSA and key establishment algorithms such as
 * Diffie-Hellman and MQV.
 *
 * If the key is not compliant then `struct ws_ffc*` will point to the
 * recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `struct ws_ffc` will also point to the recommended primitive
 * with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * **Note:** Unlike other functions in this module, this will return a
 * generic structure that specifies minimum private and public key
 * sizes.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_lenstra_validate_ffc(struct ws_context ctx, struct ws_ffc key, struct ws_ffc *alternative);

/**
 * Validates a hash function according to page 14 of the paper.
 *
 * If the hash function is not compliant then
 * `struct ws_hash* alternative` will point to the recommended
 * primitive that one should use instead.
 *
 * If the hash function is compliant but the context specifies a higher
 * security level, `struct ws_hash*` will also point to the recommended
 * primitive with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * **Note:** that this means an alternative might be suggested for a
 * compliant hash functions with a similar security level in which a
 * switch to the recommended primitive would likely be unwarranted. For
 * example, when evaluating compliance for the `SHA3-256`, a
 * recommendation to use `SHA256` will be made but this likely
 * unnecessary.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_lenstra_validate_hash(struct ws_context ctx,
                             struct ws_hash hash,
                             struct ws_hash *alternative);

/**
 * Validates  an integer factorisation cryptography primitive the most
 * common of which is the RSA signature algorithm based on pages 17-25.
 *
 * If the key is not compliant then `ws_ifc*` will point to the
 * recommended key size that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `ws_ifc*` will also point to the recommended key size with
 * the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 * **Note:** Unlike other functions in this module, this will return a
 * generic structure that specifies minimum private and public key
 * sizes.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_lenstra_validate_ifc(struct ws_context ctx, struct ws_ifc key, struct ws_ifc *alternative);

/**
 * Validates a symmetric key primitive according to pages 9-12 of the
 * paper.
 *
 * If the key is not compliant then `struct ws_symmetric* alternative`
 * will point to the recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `struct ws_symmetric*` will also point to the recommended
 * primitive with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_lenstra_validate_symmetric(struct ws_context ctx,
                                  struct ws_symmetric key,
                                  struct ws_symmetric *alternative);

/**
 * Validate an elliptic curve cryptography primitive used for digital
 * signatures and key establishment where f is the key size according
 * to page 54-55 of the standard.
 *
 * If the key is not compliant then `ws_ecc*` will contain the
 * recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `ws_ecc*` will also hold the recommended primitive with the
 * desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_nist_validate_ecc(struct ws_context ctx, struct ws_ecc key, struct ws_ecc *alternative);

/**
 * Validates a finite field cryptography primitive function examples
 * which include DSA and key establishment algorithms such as
 * Diffie-Hellman and MQV according to page 54-55 of the standard.
 *
 * If the key is not compliant then `struct ws_ffc*` will point to the
 * recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `struct ws_ffc` will also point to the recommended primitive
 * with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * **Note:** Unlike other functions in this module, this will return a
 * generic structure that specifies minimum private and public key
 * sizes.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_nist_validate_ffc(struct ws_context ctx, struct ws_ffc key, struct ws_ffc *alternative);

/**
 * Validates  an integer factorisation cryptography primitive the most
 * common of which is the RSA signature algorithm where k indicates the
 * key size according to page 54-55 of the standard.
 *
 * If the key is not compliant then `ws_ifc*` will point to the
 * recommended key size that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `ws_ifc*` will also point to the recommended key size with
 * the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 * **Note:** Unlike other functions in this module, this will return a
 * generic structure that specifies minimum private and public key
 * sizes.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_nist_validate_ifc(struct ws_context ctx, struct ws_ifc key, struct ws_ifc *alternative);

/**
 * Validates a hash function according to page 56 of the standard. The
 * reference is made with regards to applications that require
 * collision resistance such as digital signatures.
 *
 * For applications that primarily require pre-image resistance such as
 * message authentication codes (MACs), key derivation functions
 * (KDFs), and random bit generation use `ws_nist_validate_hash_based`.
 *
 * If the hash function is not compliant then
 * `struct ws_hash* alternative` will point to the recommended
 * primitive that one should use instead.
 *
 * If the hash function is compliant but the context specifies a higher
 * security level, `struct ws_hash*` will also point to the recommended
 * primitive with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * **Note:** that this means an alternative might be suggested for a
 * compliant hash functions with a similar security level in which a
 * switch to the recommended primitive would likely be unwarranted. For
 * example, when evaluating compliance for the `SHA3-256`, a
 * recommendation to use `SHA256` will be made but this likely
 * unnecessary.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_nist_validate_hash(struct ws_context ctx, struct ws_hash hash, struct ws_hash *alternative);

/**
 * Validates a hash function according to page 56 of the standard. The
 * reference is made with regards to applications that primarily
 * require pre-image resistance such as message authentication codes
 * (MACs), key derivation functions (KDFs), and random bit generation.
 *
 * For applications that require collision resistance such digital
 * signatures use `ws_nist_validate_hash`.
 *
 * If the hash function is not compliant then
 * `struct ws_hash* alternative` will point to the recommended
 * primitive that one should use instead.
 *
 * If the hash function is compliant but the context specifies a higher
 * security level, `struct ws_hash*` will also point to the recommended
 * primitive with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * **Note:** that this means an alternative might be suggested for a
 * compliant hash functions with a similar security level in which a
 * switch to the recommended primitive would likely be unwarranted. For
 * example, when evaluating compliance for the `SHA3-256`, a
 * recommendation to use `SHA256` will be made but this likely
 * unnecessary.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_nist_validate_hash_based(struct ws_context ctx,
                                struct ws_hash hash,
                                struct ws_hash *alternative);

/**
 * Validates a symmetric key primitive according to pages 54-55 of the
 * standard.
 *
 * If the key is not compliant then `struct ws_symmetric* alternative`
 * will point to the recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `struct ws_symmetric*` will also point to the recommended
 * primitive with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_nist_validate_symmetric(struct ws_context ctx,
                               struct ws_symmetric key,
                               struct ws_symmetric *alternative);

/**
 * Validate an elliptic curve cryptography primitive.
 *
 * If the key is not compliant then `ws_ecc*` will contain the
 * recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `ws_ecc*` will also hold the recommended primitive with the
 * desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_strong_validate_ecc(struct ws_context ctx, struct ws_ecc key, struct ws_ecc *alternative);

/**
 * Validates a finite field cryptography primitive.
 *
 * If the key is not compliant then `struct ws_ffc*` will point to the
 * recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `struct ws_ffc` will also point to the recommended primitive
 * with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * **Note:** Unlike other functions in this module, this will return a
 * generic structure that specifies minimum private and public key
 * sizes.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_strong_validate_ffc(struct ws_context ctx, struct ws_ffc key, struct ws_ffc *alternative);

/**
 * Validates an integer factorisation cryptography primitive the
 * most common of which is the RSA signature algorithm.
 *
 * If the key is not compliant then `ws_ifc*` will point to the
 * recommended key size that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `ws_ifc*` will also point to the recommended key size with
 * the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 * **Note:** Unlike other functions in this module, this will return a
 * generic structure that specifies minimum private and public key
 * sizes.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_strong_validate_ifc(struct ws_context ctx, struct ws_ifc key, struct ws_ifc *alternative);

/**
 * Validates a hash function.
 *
 * If the hash function is not compliant then
 * `struct ws_hash* alternative` will point to the recommended
 * primitive that one should use instead.
 *
 * If the hash function is compliant but the context specifies a higher
 * security level, `struct ws_hash*` will also point to the recommended
 * primitive with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * **Note:** that this means an alternative might be suggested for a
 * compliant hash functions with a similar security level in which a
 * switch to the recommended primitive would likely be unwarranted. For
 * example, when evaluating compliance for the `SHA3-256`, a
 * recommendation to use `SHA256` will be made but this likely
 * unnecessary.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_strong_validate_hash(struct ws_context ctx,
                            struct ws_hash hash,
                            struct ws_hash *alternative);

/**
 * Validates a symmetric key primitive.
 *
 * If the key is not compliant then `struct ws_symmetric* alternative`
 * will point to the recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `struct ws_symmetric*` will also point to the recommended
 * primitive with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_strong_validate_symmetric(struct ws_context ctx,
                                 struct ws_symmetric key,
                                 struct ws_symmetric *alternative);

/**
 * Validate an elliptic curve cryptography primitive.
 *
 * If the key is not compliant then `ws_ecc*` will contain the
 * recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `ws_ecc*` will also hold the recommended primitive with the
 * desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_weak_validate_ecc(struct ws_context ctx, struct ws_ecc key, struct ws_ecc *alternative);

/**
 * Validates a finite field cryptography primitive.
 *
 * If the key is not compliant then `struct ws_ffc*` will point to the
 * recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `struct ws_ffc` will also point to the recommended primitive
 * with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * **Note:** Unlike other functions in this module, this will return a
 * generic structure that specifies minimum private and public key
 * sizes.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_weak_validate_ffc(struct ws_context ctx, struct ws_ffc key, struct ws_ffc *alternative);

/**
 * Validates an integer factorisation cryptography primitive the
 * most common of which is the RSA signature algorithm.
 *
 * If the key is not compliant then `ws_ifc*` will point to the
 * recommended key size that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `ws_ifc*` will also point to the recommended key size with
 * the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 * **Note:** Unlike other functions in this module, this will return a
 * generic structure that specifies minimum private and public key
 * sizes.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_weak_validate_ifc(struct ws_context ctx, struct ws_ifc key, struct ws_ifc *alternative);

/**
 * Validates a hash function.
 *
 * If the hash function is not compliant then
 * `struct ws_hash* alternative` will point to the recommended
 * primitive that one should use instead.
 *
 * If the hash function is compliant but the context specifies a higher
 * security level, `struct ws_hash*` will also point to the recommended
 * primitive with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * **Note:** that this means an alternative might be suggested for a
 * compliant hash functions with a similar security level in which a
 * switch to the recommended primitive would likely be unwarranted. For
 * example, when evaluating compliance for the `SHA3-256`, a
 * recommendation to use `SHA256` will be made but this likely
 * unnecessary.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_weak_validate_hash(struct ws_context ctx, struct ws_hash hash, struct ws_hash *alternative);

/**
 * Validates a symmetric key primitive.
 *
 * If the key is not compliant then `struct ws_symmetric* alternative`
 * will point to the recommended primitive that one should use instead.
 *
 * If the key is compliant but the context specifies a higher security
 * level, `struct ws_symmetric*` will also point to the recommended
 * primitive with the desired security level.
 *
 * The function returns `1` if the hash function is compliant, `0` if
 * it is not, and `-1` if an error occurs as a result of a missing or
 * invalid argument.
 *
 * # Safety
 *
 * See crate documentation for comment on safety.
 */
int ws_weak_validate_symmetric(struct ws_context ctx,
                               struct ws_symmetric key,
                               struct ws_symmetric *alternative);

#ifdef __cplusplus
} // extern "C"
#endif // __cplusplus

#endif /* WARDSTONE_H_ */
